@@ -1,5 +1,6 @@
 /***************************************************************************
-* Copyright (c) 2016, Wolf Vollprecht, Johan Mabille and Sylvain Corlay    *
+* Copyright (c) Wolf Vollprecht, Johan Mabille and Sylvain Corlay          *
+* Copyright (c) QuantStack                                                 *
 *                                                                          *
 * Distributed under the terms of the BSD 3-Clause License.                 *
 *                                                                          *
@@ -9,9 +10,9 @@
 #ifndef XBLAS_CONFIG_CLING_HPP
 #define XBLAS_CONFIG_CLING_HPP
 
-#pragma cling add_library_path("C:/bld/xtensor-blas_1556539354639/_h_env/Library/lib64")
-#pragma cling add_library_path("C:/bld/xtensor-blas_1556539354639/_h_env/Library/lib32")
-#pragma cling add_library_path("C:/bld/xtensor-blas_1556539354639/_h_env/Library/lib")
+#pragma cling add_library_path("D:/bld/xtensor-blas_1576020268187/_h_env/Library/lib64")
+#pragma cling add_library_path("D:/bld/xtensor-blas_1576020268187/_h_env/Library/lib32")
+#pragma cling add_library_path("D:/bld/xtensor-blas_1576020268187/_h_env/Library/lib")
 
 #ifndef XTENSOR_USE_FLENS_BLAS
 
