@@ -1,5 +1,6 @@
 /***************************************************************************
-* Copyright (c) 2016, Wolf Vollprecht, Johan Mabille and Sylvain Corlay    *
+* Copyright (c) Wolf Vollprecht, Johan Mabille and Sylvain Corlay          *
+* Copyright (c) QuantStack                                                 *
 *                                                                          *
 * Distributed under the terms of the BSD 3-Clause License.                 *
 *                                                                          *
@@ -260,7 +261,7 @@ namespace lapack
         std::size_t n = A.shape()[1];
 
         xtype1 s;
-        s.resize({ std::max(std::size_t(1), std::min(m, n)) });
+        s.resize({ std::max(static_cast<std::size_t>(1), std::min(m, n)) });
 
         xtype2 u, vt;
 
@@ -348,7 +349,7 @@ namespace lapack
         }
 
         xtype1 s;
-        s.resize({ std::max(std::size_t(1), std::min(m, n)) });
+        s.resize({ std::max(static_cast<std::size_t>(1), std::min(m, n)) });
 
         xtype2 u, vt;
 
@@ -446,7 +447,7 @@ namespace lapack
             throw std::runtime_error("Could not find workspace size for getri.");
         }
 
-        work.resize(std::size_t(std::real(work[0])));
+        work.resize(static_cast<std::size_t>(std::real(work[0])));
 
         info = cxxlapack::getri<blas_index_t>(
             static_cast<blas_index_t>(A.shape()[0]),
@@ -496,7 +497,7 @@ namespace lapack
             throw std::runtime_error("Could not find workspace size for geev.");
         }
 
-        work.resize(std::size_t(work[0]));
+        work.resize(static_cast<std::size_t>(work[0]));
 
         info = cxxlapack::geev<blas_index_t>(
             jobvl,
@@ -570,6 +571,69 @@ namespace lapack
         return info;
     }
 
+    /**
+     * Interface to LAPACK sygvd.
+     * @returns info
+     */
+    template <class E, class W>
+    int sygvd(E& A, E& B, blas_index_t itype, char jobz, char uplo, W& w)
+    {
+        XTENSOR_ASSERT(A.dimension() == 2);
+        XTENSOR_ASSERT(A.layout() == layout_type::column_major);
+        XTENSOR_ASSERT(B.dimension() == 2);
+        XTENSOR_ASSERT(B.layout() == layout_type::column_major);
+
+        using value_type = typename E::value_type;
+
+        auto N = A.shape()[0];
+        XTENSOR_ASSERT(B.shape()[0] ==N);
+
+        uvector<value_type> work(1);
+        uvector<blas_index_t> iwork(1);
+
+        int info = cxxlapack::sygvd<blas_index_t>(
+            itype,
+            jobz,
+            uplo,
+            static_cast<blas_index_t>(N),
+            A.data(),
+            stride_back(A),
+            B.data(),
+            stride_back(B),
+            w.data(),
+            work.data(),
+            static_cast<blas_index_t>(-1),
+            iwork.data(),
+            static_cast<blas_index_t>(-1)
+        );
+
+        if (info != 0)
+        {
+            throw std::runtime_error("Could not find workspace size for sygvd.");
+        }
+
+        work.resize(std::size_t(work[0]));
+        iwork.resize(std::size_t(iwork[0]));
+
+        info = cxxlapack::sygvd<blas_index_t>(
+            itype,
+            jobz,
+            uplo,
+            static_cast<blas_index_t>(N),
+            A.data(),
+            stride_back(A),
+            B.data(),
+            stride_back(B),
+            w.data(),
+            work.data(),
+            static_cast<blas_index_t>(work.size()),
+            iwork.data(),
+            static_cast<blas_index_t>(iwork.size())
+        );
+
+        return info;
+    }
+
     /**
      * Complex version of geev
      */
@@ -699,6 +763,7 @@ namespace lapack
         std::size_t m = A.shape()[0];
         std::size_t n = A.shape()[1];
 
+        blas_index_t a_stride = static_cast<blas_index_t>(std::max(std::size_t(1), m));
         blas_index_t b_stride = static_cast<blas_index_t>(std::max(std::max(std::size_t(1), m), n));
 
         int info = cxxlapack::gelsd<blas_index_t>(
@@ -706,7 +771,7 @@ namespace lapack
             static_cast<blas_index_t>(A.shape()[1]),
             b_dim,
             A.data(),
-            stride_back(A),
+            a_stride,
             b.data(),
             b_stride,
             s.data(),
@@ -730,7 +795,7 @@ namespace lapack
             static_cast<blas_index_t>(A.shape()[1]),
             b_dim,
             A.data(),
-            stride_back(A),
+            a_stride,
             b.data(),
             b_stride,
             s.data(),
