@@ -1,5 +1,6 @@
 /***************************************************************************
-* Copyright (c) 2016, Wolf Vollprecht, Johan Mabille and Sylvain Corlay    *
+* Copyright (c) Wolf Vollprecht, Johan Mabille and Sylvain Corlay          *
+* Copyright (c) QuantStack                                                 *
 *                                                                          *
 * Distributed under the terms of the BSD 3-Clause License.                 *
 *                                                                          *
@@ -20,6 +21,7 @@
 #include "xtensor/xcomplex.hpp"
 #include "xtensor/xeval.hpp"
 #include "xtensor/xio.hpp"
+#include "xtensor/xmanipulation.hpp"
 #include "xtensor/xstrided_view.hpp"
 #include "xtensor/xtensor.hpp"
 #include "xtensor/xutils.hpp"
@@ -94,7 +96,7 @@ namespace linalg
                 {
                     result += std::abs(std::pow(v(i), ord));
                 }
-                result = std::pow(result, 1./ double(ord));
+                result = std::pow(result, 1. / static_cast<double>(ord));
             }
             return result;
         }
@@ -150,7 +152,7 @@ namespace linalg
         {
             if (ord == normorder::frob)
             {
-                return std::sqrt(sum(pow(abs(v), 2))());
+                return static_cast<value_type>(std::sqrt(sum(pow(abs(v), 2))()));
             }
             if (ord == normorder::nuc)
             {
@@ -352,7 +354,7 @@ namespace linalg
         std::array<std::size_t, 1> vN = {N};
         xtensor<value_type, 1, layout_type::column_major> w(vN);
 
-        std::array<std::size_t, 2> shp({N, N});
+        std::array<std::size_t, 2> shp = {N, N};
         xtensor<value_type, 2, layout_type::column_major> VL(shp), VR(shp);
 
         // jobvl N: left eigenvectors of A are not computed
@@ -414,6 +416,57 @@ namespace linalg
         return std::make_tuple(std::move(w), std::move(M));
     }
 
+    /**
+     * Compute the generalized eigenvalues and eigenvectors of a square Hermitian or real symmetric
+     * xexpression.
+     *
+     * @param A,B Matrices for which the generalized eigenvalues and right eigenvectors are computed
+     * @return xtensor containing the eigenvalues.
+     */
+    template <class E, std::enable_if_t<!xtl::is_complex<typename E::value_type>::value>* = nullptr>
+    auto eigh(const xexpression<E>& A, const xexpression<E>& B,const char UPLO = 'L')
+    {
+        using value_type = typename E::value_type;
+
+        auto M1 = copy_to_layout<layout_type::column_major>(A.derived_cast());
+        auto M2 = copy_to_layout<layout_type::column_major>(B.derived_cast());
+
+        std::size_t N = M1.shape()[0];
+        std::array<std::size_t, 1> vN = {N};
+        xtensor<value_type, 1, layout_type::column_major> w(vN);
+
+        int info = lapack::sygvd(M1, M2, 1, 'V', UPLO, w);
+        if (info != 0)
+        {
+            throw std::runtime_error("Eigenvalue computation did not converge.");
+        }
+
+        return std::make_tuple(std::move(w), std::move(M1));
+    }
+
+    #if 0
+    template <class E, std::enable_if_t<xtl::is_complex<typename E::value_type>::value>* = nullptr>
+    auto eigh(const xexpression<E>& A, char UPLO = 'L')
+    {
+        using value_type = typename E::value_type;
+        using underlying_value_type = typename value_type::value_type;
+
+        auto M = copy_to_layout<layout_type::column_major>(A.derived_cast());
+
+        std::size_t N = M.shape()[0];
+        std::array<std::size_t, 1> vN = {N};
+        xtensor<underlying_value_type, 1, layout_type::column_major> w(vN);
+
+        int info = lapack::heevd(M, 'V', UPLO, w);
+        if (info != 0)
+        {
+            throw std::runtime_error("Eigenvalue computation did not converge.");
+        }
+
+        return std::make_tuple(std::move(w), std::move(M));
+    }
+    #endif
+
     /**
      * Compute the eigenvalues of a square xexpression.
      *
@@ -433,11 +486,16 @@ namespace linalg
         xtensor<value_type, 1, layout_type::column_major> wi(vN);
 
         // TODO check if we can remove allocation and pass nullptr as VL / VR
-        std::array<std::size_t, 2> shp({N, N});
+        std::array<std::size_t, 2> shp = {N, N};
         xtensor<value_type, 2, layout_type::column_major> VL(shp);
         xtensor<value_type, 2, layout_type::column_major> VR(shp);
 
-        auto geev_res = lapack::geev(M, 'N', 'N', wr, wi, VL, VR);
+        auto info = lapack::geev(M, 'N', 'N', wr, wi, VL, VR);
+        if (info != 0)
+        {
+            throw std::runtime_error("Failed to compute eigenvalue " +
+                std::to_string(std::abs(info)) + ".");
+        }
 
         xtensor<std::complex<value_type>, 1> eig_vals;
         eig_vals.resize({N});
@@ -460,11 +518,16 @@ namespace linalg
         xtensor<value_type, 1, layout_type::column_major> w(vN);
 
         // TODO check if we can remove allocation and pass nullptr as VL / VR
-        std::array<std::size_t, 2> shp({N, N});
+        std::array<std::size_t, 2> shp = {N, N};
         xtensor<value_type, 2, layout_type::column_major> VL(shp);
         xtensor<value_type, 2, layout_type::column_major> VR(shp);
 
-        auto geev_res = lapack::geev(M, 'N', 'N', w, VL, VR);
+        auto info = lapack::geev(M, 'N', 'N', w, VL, VR);
+        if (info != 0)
+        {
+            throw std::runtime_error("Failed to compute eigenvalue " +
+                std::to_string(std::abs(info)) + ".");
+        }
 
         using value_type = typename E::value_type;
 
@@ -532,12 +595,14 @@ namespace linalg
                 : m_a(a), m_axis(axis)
             {
                 resize_container(m_idx, a.dimension());
+                std::fill(m_idx.begin(), m_idx.end(), 0);
                 m_offset = 0;
             }
 
+            #define SC(X) static_cast<std::size_t>(X)
             inline bool next()
             {
-                size_type dim = m_a.dimension();
+                size_type dim = static_cast<size_type>(m_a.dimension());
                 for (size_type j = dim; j != 0; --j)
                 {
                     size_type i = j - 1;
@@ -545,9 +610,9 @@ namespace linalg
                     {
                         // skip
                     }
-                    else if (m_idx[i] == m_a.shape()[i] - 1)
+                    else if (m_idx[SC(i)] == m_a.shape()[SC(i)] - 1)
                     {
-                        m_offset -= m_idx[i] * static_cast<size_type>(m_a.strides()[i]);
+                        m_offset -= static_cast<std::ptrdiff_t>(m_idx[i]) * m_a.strides()[i];
                         m_idx[i] = size_type(0);
                         if (i == 0 || m_axis == 0 && i == 1)
                         {
@@ -557,14 +622,14 @@ namespace linalg
                     else
                     {
                         ++m_idx[i];
-                        m_offset += static_cast<size_type>(m_a.strides()[i]);
+                        m_offset += m_a.strides()[i];
                         return true;
                     }
                 }
                 return false;
             }
-
-            inline size_type offset() const
+            #undef SC
+            inline std::ptrdiff_t offset() const
             {
                 return m_offset;
             }
@@ -573,7 +638,7 @@ namespace linalg
             const A& m_a;
             index_type m_idx;
             size_type m_axis;
-            size_type m_offset;
+            std::ptrdiff_t m_offset;
         };
     }
 
@@ -605,6 +670,11 @@ namespace linalg
         auto&& t = view_eval<T::static_layout>(xt.derived_cast());
         auto&& o = view_eval<O::static_layout>(xo.derived_cast());
 
+        // is one of each a scalar? just multiply
+        if (t.dimension() == 0 || o.dimension() == 0)
+        {
+            return return_type(t * o);
+        }
         if (t.dimension() == 1 && o.dimension() == 1)
         {
             result.resize(std::vector<std::size_t>{1});
@@ -1343,7 +1413,7 @@ namespace linalg
         XTENSOR_ASSERT(da.dimension() == 2);
         XTENSOR_ASSERT(db.dimension() == 2);
 
-        std::array<std::size_t, 2> shp = { da.shape()[0] * db.shape()[0], da.shape()[1] * db.shape()[1] };
+        std::array<std::size_t, 2> shp = {da.shape()[0] * db.shape()[0], da.shape()[1] * db.shape()[1]};
         xtensor<value_type, 2> res(shp);
 
         for (std::size_t i = 0; i < da.shape()[0]; ++i)
@@ -1696,7 +1766,7 @@ namespace linalg
         for (std::size_t i = 0; i < b.dimension(); ++i)
         {
             auto b_ax_it = std::find(ax_b.begin(), ax_b.end(), i);
-            // seccond pass if i is not in ax_b add to newaxes_b
+            // second pass if i is not in ax_b add to newaxes_b
             if (b_ax_it == ax_b.end())
             {
                 newaxes_b.push_back(i);
